@@ -101,6 +101,7 @@ extern SEXP nano_TextframesSymbol;
 extern SEXP nano_UnserSymbol;
 extern SEXP nano_UrlSymbol;
 
+extern SEXP nano_addRedirect;
 extern SEXP nano_aioFormals;
 extern SEXP nano_aioFuncs;
 extern SEXP nano_aioNFuncs;
@@ -127,6 +128,7 @@ extern SEXP rnng_device(SEXP, SEXP);
 extern SEXP rnng_dial(SEXP, SEXP, SEXP);
 extern SEXP rnng_dialer_close(SEXP);
 extern SEXP rnng_dialer_start(SEXP, SEXP);
+extern SEXP rnng_get_opt(SEXP, SEXP);
 extern SEXP rnng_listen(SEXP, SEXP, SEXP);
 extern SEXP rnng_listener_close(SEXP);
 extern SEXP rnng_listener_start(SEXP);
@@ -140,7 +142,7 @@ extern SEXP rnng_recv_aio(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP rnng_request(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP rnng_send(SEXP, SEXP, SEXP, SEXP);
 extern SEXP rnng_send_aio(SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP rnng_set_opt(SEXP, SEXP, SEXP, SEXP);
+extern SEXP rnng_set_opt(SEXP, SEXP, SEXP);
 extern SEXP rnng_sha224(SEXP, SEXP, SEXP);
 extern SEXP rnng_sha256(SEXP, SEXP, SEXP);
 extern SEXP rnng_sha384(SEXP, SEXP, SEXP);
@@ -151,6 +153,7 @@ extern SEXP rnng_stream_close(SEXP);
 extern SEXP rnng_stream_dial(SEXP, SEXP, SEXP);
 extern SEXP rnng_stream_listen(SEXP, SEXP, SEXP);
 extern SEXP rnng_strerror(SEXP);
+extern SEXP rnng_subscribe(SEXP, SEXP, SEXP);
 extern SEXP rnng_thread_create(SEXP);
 extern SEXP rnng_unresolved(SEXP);
 extern SEXP rnng_version(void);
